/**
 * dhwk kernel driver
 * Copyright (C) 2006:
 *   Daniel -Blackamp- Michalik
 *   Sebastian tokkee Harl
 *   Florian octo Forster
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 **/

#ifndef DHWK_H
#define DHWK_H 1

#define DHWK_MODULE_NAME "dhwk"

#ifndef DHWK_MAJOR
#define DHWK_MAJOR 0 /* dynamic */
#endif

#define DHWK_ADDR_SEND 0x300
#define DHWK_ADDR_RECV 0x302
#define DHWK_ADDR_INTR 0x304 /* Interrupt register */
#define DHWK_ADDR_INTM 0x306 /* Interrupt mask */
#define DHWK_ADDR_MGIC 0x308

#define DHWK_INTR_SOFTIRQ 	(0x0001 << 0)
#define DHWK_INTR_RECV_EF_UNSET (0x0001 << 1)
#define DHWK_INTR_RECV_HF_ISSET (0x0001 << 2)
#define DHWK_INTR_RECV_FF_ISSET (0x0001 << 3)
#define DHWK_INTR_SEND_ERROR    (0x0001 << 4)
#define DHWK_INTR_SEND_EF_ISSET (0x0001 << 5)
#define DHWK_INTR_SEND_HF_UNSET (0x0001 << 6)
#define DHWK_INTR_SEND_FF_UNSET (0x0001 << 7)

#define DHWK_ENABLE_RECV 0x0400
#define DHWK_ENABLE_SEND 0x0200

#define DHWK_SOFTIRQ 0x0801

#define DHWK_RESET 0x0100

#define DHWK_GROUP   0x03
#define DHWK_MONTH   0x03
#define DHWK_DAY     0x08
#define DHWK_VERSION 0x01

#define DHWK_MAGIC (DHWK_GROUP << 12) | (DHWK_MONTH << 8) | (DHWK_DAY << 3) | DHWK_VERSION

#define DHWK_BUFFER_SIZE 4096
#define DHWK_BLOCK_SIZE 1024

#define DHWK_BUFFER_FULL_SIZE 4096
#define DHWK_BUFFER_HALF_SIZE 2048

#define DHWK_MAX_READ  1
#define DHWK_MAX_WRITE 1

#define DHWK_RECV_EF(f) (!(((f) >>  9) & 0x01))
#define DHWK_RECV_HF(f) (!(((f) >> 10) & 0x01))
#define DHWK_RECV_FF(f) (!(((f) >> 11) & 0x01))

#define DHWK_SEND_EF(f) (!(((f) >> 13) & 0x01))
#define DHWK_SEND_HF(f) (!(((f) >> 14) & 0x01))
#define DHWK_SEND_FF(f) (!(((f) >> 15) & 0x01))

#define DHWK_SEND_ENABLED(f) (((f) >> 9) & 0x01)
#define DHWK_RECV_ENABLED(f) (((f) >> 10) & 0x01)

#define DHWK_IS_NONBLOCK(fp) ((fp)->f_flags & O_NONBLOCK)

#define DHWK_IOCTL_GETGROUP	0x0001
#define DHWK_IOCTL_SOFTIRQ	0x0002
#define DHWK_IOCTL_CHANGE_SEND	0x0004
#define DHWK_IOCTL_CHANGE_RECV	0x0008
#define DHWK_IOCTL_RESET	0x0010

typedef struct
{
	/* Do _not_ _ever_ use the locks in an IRQ handler! */
	spinlock_t lock;
	wait_queue_head_t wait;
	char       data[DHWK_BUFFER_SIZE];
	int        pos_read;
	int        pos_write;
	int	   ready;
} dhwk_buffer_t;

typedef struct
{
	int major;
	int interrupt;
	int open_count_read;
	int open_count_write;
	dhwk_buffer_t *buffer_recv;
	dhwk_buffer_t *buffer_send;
	struct tasklet_struct tasklet;
	struct proc_dir_entry *proc_dir;
	struct proc_dir_entry *proc_recv;
	struct proc_dir_entry *proc_send;
	struct proc_dir_entry *proc_info;
} dhwk_data_t;

#endif /* DHWK_H */
