#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <sys/select.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#define DEV "dev_dhwk"
#define DHWK_IOCTL_GETGROUP     0x0001
#define DHWK_IOCTL_SOFTIRQ      0x0002
#define DHWK_IOCTL_CHANGE_SEND  0x0004
#define DHWK_IOCTL_CHANGE_RECV  0x0008
#define DHWK_IOCTL_RESET	0x0010


int main( int argc, char **argv) {
	int fd;
	char buf[128];
	char*bufptr;
	int  buflen;
	struct pollfd spfd;
	int i;
	fd_set writefds;
	
	memset(buf, 0, sizeof(buf));

	fd = open(DEV, O_RDWR | O_NONBLOCK);
	if(fd < 0){
		fprintf (stderr, "open (%s) failed: %s",
				DEV, strerror (errno));
		return -1;
	}

	ioctl(fd, DHWK_IOCTL_GETGROUP, buf);
	printf("DHWK_IOCTL_GETGROUP: `%s'\n", buf);
	ioctl(fd, DHWK_IOCTL_SOFTIRQ, 0L);
	/* Just to show that it works... */
	ioctl(fd, DHWK_IOCTL_CHANGE_RECV, 0);
	ioctl(fd, DHWK_IOCTL_CHANGE_SEND, 0);
	ioctl(fd, DHWK_IOCTL_CHANGE_RECV, 1);
	ioctl(fd, DHWK_IOCTL_CHANGE_SEND, 1);

	ioctl(fd, DHWK_IOCTL_RESET, 0);

	memset(buf, 0, sizeof(buf));
	strncpy(buf, "Hello world!", sizeof(buf) - 1);

	/*
	spfd.fd = fd;
	spfd.events = POLLOUT;
	spfd.revents = 0;

	printf ("Calling poll on 1 fd\n");
	if (poll (&spfd, 1, -1) <= 0)
	{
		perror ("poll");
		return 1;
	}
	printf ("poll returned\n");
	*/

	FD_ZERO(&writefds);
	FD_SET(fd, &writefds);

	printf ("Calling poll on 1 fd\n");
	if (select(fd + 1, NULL, &writefds, NULL, NULL) <= 0)
	{
		perror ("select");
		return 1;
	}
	printf ("select returned\n");

	buflen = strlen (buf);
	bufptr = buf;
	while (buflen > 0)
	{
		fprintf (stderr, "Line %i: pre  write\n", __LINE__);
		i = write(fd, bufptr, buflen);
		fprintf (stderr, "Line %i: post write\n", __LINE__);

		if (i < 0)
		{
			perror ("write");
			return (1);
		}

		buflen -= i;
		bufptr += i;
		assert (buflen >= 0);
	}

	buflen = sizeof (buf) - 1;
	bufptr = buf;
	fprintf (stderr, "Line %i: pre  read\n", __LINE__);
	while (1)
	{
		i = read (fd, bufptr, buflen);
		//fprintf (stderr, "Line %i: read (...) = %i\n", __LINE__, i);

		if ((i == -1) && (errno == EAGAIN))
		{
			continue;
		}
		else if (i <= 0)
		{
			break;
		}

		assert ((i > 0) && (i <= buflen));
		fprintf (stderr, "Line %i: post read\n", __LINE__);

		bufptr += i;
		buflen -= i;
		break;
	} /* while */
	assert (buflen < sizeof (buf));

	if (i < 0)
	{
		perror ("read");
		return (1);
	}
	else
	{
		buf[sizeof (buf) - (buflen + 1)] = '\0';
		printf ("Successfully read %i bytes: `%s'\n",
				sizeof (buf) - (buflen + 1), buf);
	}

	close(fd);


	fd = open(DEV, O_RDWR);
	printf("Waiting for data..\n");

	spfd.fd = fd;
	spfd.events = POLLIN;
	spfd.revents = 0;

	if (poll (&spfd, 1, -1) <= 0)
	{
		perror ("poll");
		return 1;
	}
	printf ("poll returned\n");

	i = read(fd, buf, sizeof(buf) - 1);
	if(i <= 0) {
		perror("read()");
	}
	buf[i] = '\0';
	
	printf ("Successfully read %i bytes: `%s'\n", i, buf);

	close(fd);
	return 0;
}

